//******************************************************************************
// This class is part of the Sizmek 360 Video Ad Feature
// ALL RIGHTS RESERVED TO Sizmek.
//******************************************************************************
//******************************************************************************
(function() {
	// Define Method
	window.ThreeSixtyViewer = function(_c) {

		"use strict";
		// Verify that Adkit
		if (document.location.protocol !== "file:" && (typeof EB === "undefined" || typeof EBG === "undefined")) {
			// Console error
			console.log('Adkit not ready');
			// abandon script
			return false;
		}
		// Save Reference to Script Name
		var scriptName = "Video360";
		// Script Version
		var scriptVersion = "1.0.3";
		// Last Modified
		var lastModified = "2020-07-27";
		// Last Uploaded
		var lastUploaded = "2020-07-27";
		// Template Version
		var templateVersion = "2.0.7";
		// Establish Boolean for tracking
		var isDebug = false;
		// Establish Boolean for tracking
		var trackOnce = true;
		// Establish reference to return obj
		var featureAPI;
		// Establish reference to track which scene is loaded
		var theScene;
		// Establish reference to video element
		var theVideo = null;
		// Establish reference to tracked video module
		var trackedVideo;
		// Get Video Source from config
		var videoSource = _c.videoSource || '';
		// Get loop preference from config
		var videoLoop = _c.videoLoop;
		// Get mute preference from config
		var videoMuted = _c.videoMuted;
		// Establish Reference to image source
		var imageSource = _c.imageSource || '';
		// Establish reference to initial source
		var initSource = _c.initSource || 'image';
		// Establish boolean for play status
		var videoPlaying = _c.videoAutoPlay;
		// Establish initial rotation position
		var rotateTo = _c.rotateTo || "0.126 -0.189 4.99";
		// Establish reference for auto pan preference
		var enableAutoPan = _c.enableAutoPan || false;
		// Establish reference for pan controls preference
		var enableVideoControls = _c.enableVideoControls || false;
		// Establish reference for pan controls preference
		var enablePanControls = _c.enablePanControls || false;
		// Establish reference for pan controls preference
		var enableKeyControls = _c.enableKeyControls || false;
		// Establish reference for field of view preference
		var fov = _c.fieldOfView || 75;
		// Establish reference to video container
		var container = document.getElementById(_c.container);
		// Establish references for widths and heights
		var containerW, containerH;
		// Establish an object to store styles
		var previousStyling = {};
		// Establish references for three.js camera, scene and renderer
		var aScene;
		// Establish boolean to track if in fullscreen
		var isFS = false;
		// Establish boolean to track if in VR
		var inVR = false;
		// Establish boolean to track Mobile
		var isMobile = false;
		// Establish boolean to track if in fullscreen
		var panning = false;
		// Establish boolean to track if video controls exist
		var videoControls = false;
		// Establish timeout var
		var holdPanTimeout;
		// Establish int to track viewed scene
		var viewPercent = 0;
		// Establish Booleans to track how much has been viewed
		var viewComplete = false;
		var view25Percent = false;
		var view50Percent = false;
		var view75Percent = false;
		// Establish var for positioning
		var lon = 0;
		var lat = 0;
		var phi = 0;
		var theta = 0;

		// Function That Creates Element Var
		var d = function(_id) {
			return document.getElementById(_id);
		};
		// Function That Creates selector Var
		var q = function(_id) {
			return document.querySelector(_id);
		};
		// Create Event Handlers for class
		var addEventListener = function(_e, _m, _b) {
			window.addEventListener(_e, _m, _b);
		};
		// Remove Event Handlers for class
		var removeEventListener = function(_e, _m, _b) {
			window.removeEventListener(_e, _m, _b);
		};

		var init = function() {
			// Check if IE11, stop and throw unsupported event
			if (!!navigator.userAgent.match(/Trident/g)) {
				// Fire event for play
				dispatchEvent("notSupported", {
					'threeSixtyViewerAPI': featureAPI
				});
			}
			// reset lat/long
			lat = lon = 0;
			// Get Width of container
			containerW = container.offsetWidth;
			// Get Height of container
			containerH = container.offsetHeight;
			// Check if video option exists, create video tag
			if (videoSource !== '') {
				buildVideoTag();
			}

			// Establish if mobile
			if (EB.API && EB.API.os) {
				isMobile = EB.API.os.mobile;
			}
			// Check if initial source is set to video
			if (initSource === "video") {
				// load 360 video
				doLoadVideo();
			}
			else {
				// load 360 image
				doLoadImage();
			}

			if (enableAutoPan) {
				autoPan();
			}
			// Add listeners
			addListeners();
			// reset view Vars
			resetViewPercent();
		};

		var loadImage = function() {
			// Update boolean
			initSource = "image";
			// check if scene exists
			if (theScene !== "#sky") {
				// remove scene
				destroy();
				// add scene
				init();
			}
		};

		var doLoadImage = function() {
			// Build Controls
			buildControls();
			// Create new AFrame scene element
			aScene = document.createElement("a-scene");
			// Make the scene an embedded scene
			aScene.setAttribute('embedded', '');
			// Hide scene till ready
			aScene.style.opacity = 0;
			// Establish reference to entity
			var sky;
			// only need pan controls for desktop
			if (!isMobile) {
				// Create new AFrame Camera element
				var camera = '<a-entity position="0 0 1">';
				camera += '<a-camera id="camera" fov="' + fov + '" ';
				camera += 'orbit-controls="target: #target; enableDamping: true; dampingFactor: 0.125; rotateSpeed:0.25; rotateToSpeed: 1; rotateTo: ' + rotateTo + ';">';
				camera += '</a-camera></a-entity>';

				camera += '<a-entity id="target" position="0 0 0">';
				camera += '<a-sky id="target"></a-sky>';
				camera += '</a-entity>';
				aScene.innerHTML = camera;
				// Add scene to container
				container.appendChild(aScene);
				// Get reference to sky entity
				sky = q("a-sky");
			}
			else {
				// Add scene to container
				container.appendChild(aScene);
				// Create new sky element
				sky = document.createElement("a-sky");
				// Make the scene an embedded scene
				sky.setAttribute("id", "sky");
				// Add Sky element to scene
				aScene.appendChild(sky);
			}
			// Update scene source with image
			sky.setAttribute('src', imageSource);
			// Track type of scene
			theScene = "#sky";
			// Add Scene listeners
			addSceneListeners();
			// show container since first load it will be hidden
			setTimeout(function() {
				// show scene
				aScene.style.opacity = 1;
				// Show Pan controls
				if (!isMobile) {
					d("panControls").style.display = "block";
				}
				// Calling stop drag updates pan positioning
				stopDrag();
			}, 1000);
		};

		var loadVideo = function() {
			// Check if IE11, stop and throw unsupported event
			if (!!navigator.userAgent.match(/Trident/g)) {
				// Fire event for play
				dispatchEvent("notSupported", {
					'threeSixtyViewerAPI': featureAPI
				});
				return;
			}
			// Update boolean
			initSource = "video";
			// check if scene exists
			if (theScene !== "#videosphere") {
				// remove scene
				destroy();
				// add scene
				init();
			}
		};

		// Handle 360 video creation
		var doLoadVideo = function() {
			// Build Controls
			buildControls();
			// check video control preference
			if (enableVideoControls) {
				// add controls
				buildVideoControls();
			}
			else {
				log('Current setup includes no video controls');
			}
			// Create new AFrame scene element
			aScene = document.createElement("a-scene");
			// Make the scene an embedded scene
			aScene.setAttribute('embedded', '');
			// Hide scene till ready
			aScene.style.opacity = 0;
			// Establish reference to video entity
			var aVideosphere;
			// only need pan controls on desktop
			if (!isMobile) {
				// Create new AFrame Camera element
				var camera = '<a-entity position="0 0 1">';
				camera += '<a-camera id="camera" fov="' + fov + '" ';
				camera += 'orbit-controls="target: #target; enableDamping: true; dampingFactor: 0.125; rotateSpeed:0.25; rotateToSpeed: 1; rotateTo: ' + rotateTo + ';">';
				camera += '</a-camera></a-entity>';

				camera += '<a-entity id="target" position="0 0 0">';
				camera += '<a-videosphere id="target"></a-videosphere>';
				camera += '</a-entity>';
				// Add html to scene
				aScene.innerHTML = camera;
				// Add scene to container
				container.appendChild(aScene);
				// Get reference to videosphere
				aVideosphere = q("a-videosphere");
			}
			else {
				// Add scene to container
				container.appendChild(aScene);
				// Create new AFrame videosphere element
				aVideosphere = document.createElement("a-videosphere");
				// update rotation attribute
				aVideosphere.setAttribute("id", "videosphere");
				// Add Sky element to scene
				aScene.appendChild(aVideosphere);
			}
			// Create new AFrame assets element
			var aAssets = document.createElement("a-assets");
			// Add video to assets
			aAssets.appendChild(theVideo);
			// Add Asset Entity to Scene
			aScene.appendChild(aAssets);
			// Update VideoSphere source
			aVideosphere.setAttribute("src", "#video");
			// Track type of scene
			theScene = "#videosphere";
			// Add Scene listeners
			addSceneListeners();
			// show container since first load it will be hidden
			setTimeout(function() {
				// show scene
				aScene.style.opacity = 1;
				// Show Pan controls
				if (!isMobile) {
					d("panControls").style.display = "block";
				}
				// check play status
				if (videoPlaying !== false && videoPlaying !== undefined) {
					// Play Video
					trackedVideo.playVideo();
				}
				else {
					handleOnPause();
				}
				// Calling stop drag updates pan positioning
				stopDrag();
			}, 1000);
		};

		var buildVideoTag = function() {
			// Create new video
			theVideo = document.createElement("video");
			// Set the Id of the video tag
			theVideo.setAttribute("id", "video");
			// Update video source
			loadVideoSource();
			// Create video module instance to track video events
			trackedVideo = new EBG.VideoModule(theVideo);
			// update domain policy
			theVideo.crossOrigin = 'anonymous';
			// update width from container
			theVideo.width = containerW;
			// update height from container
			theVideo.height = containerH;
			// Check video loop preference
			if (videoLoop !== undefined) {
				// Set loop preference from config
				theVideo.loop = videoLoop;
			}
			// Set playsline attribute
			theVideo.setAttribute("playsinline", "");
			// Set playsline attribute - should be deprecated but no
			theVideo.setAttribute("webkit-playsinline", "");
			// Check video loop preference
			if (videoMuted !== undefined) {
				// Set mute preference from config
				theVideo.muted = videoMuted;
			}
			// make sure we hide video
			theVideo.style.display = "none";
		};

		// Add video source to video tag
		var loadVideoSource = function() {
			// Create Source Element
			var src;
			// Check if video source is an array
			if (Array.isArray(videoSource)) {
				// loop through source array
				for (var i = 0; i < videoSource.length; i++) {
					// Create Source Element
					src = document.createElement('source');
					// Update Video Source
					src.src = videoSource[i][0];
					// Update Video Type
					src.type = videoSource[i][1];
					// Append Video Source To Video Element
					theVideo.appendChild(src);
				}
			}
			else {
				// Create Source Element
				src = document.createElement('source');
				// Set video source from config
				theVideo.src = videoSource;
			}
		};

		// build and add video controls
		var buildVideoControls = function() {
			// update boolean
			videoControls = true;
			// create new div
			var myContainerDiv = document.createElement("div");
			// Add id to div
			myContainerDiv.id = "videoControls";
			// Add to container
			container.appendChild(myContainerDiv);
			// Create new item
			var myDiv = document.createElement("div");
			// add id
			myDiv.id = "videoPlayButton";
			// Add Class
			myDiv.className = "videoPlayButton";
			// add title to item
			myDiv.title = "Play Video";
			// add listener
			myDiv.addEventListener("click", function() {
				// track play status
				videoPlaying = true;
				// play video
				playVideo();
			}, false);
			// add item to div
			myContainerDiv.appendChild(myDiv);
			// Create new item
			myDiv = document.createElement("div");
			// add id
			myDiv.id = "videoPauseButton";
			// Add Class
			myDiv.className = "videoPauseButton";
			// add title to item
			myDiv.title = "Pause Video";
			// add title to item
			myDiv.style.display = "none";
			// add listener
			myDiv.addEventListener("click", function() {
				// track play status
				videoPlaying = false;
				// play video
				pauseVideo();
			}, false);
			// add item to div
			myContainerDiv.appendChild(myDiv);
			// Create new item
			myDiv = document.createElement("div");
			// add id
			myDiv.id = "videoMuteButton";
			// Add Class
			myDiv.className = "videoMuteButton";
			// add title to item
			myDiv.title = "Mute Video";
			// add listener
			myDiv.addEventListener("click", mute, false);
			// add item to div
			myContainerDiv.appendChild(myDiv);
			// Create new item
			myDiv = document.createElement("div");
			// add id
			myDiv.id = "videoUnMuteButton";
			// Add Class
			myDiv.className = "videoUnMuteButton";
			// add title to item
			myDiv.title = "UnMute Video";
			// add listener
			myDiv.addEventListener("click", unMute, false);
			// add item to div
			myContainerDiv.appendChild(myDiv);
			// check if muted
			if (!theVideo.muted) {
				// hide unmute button
				d("videoUnMuteButton").style.display = "none";
			}
			else {
				// hide mute button
				d("videoMuteButton").style.display = "none";
			}
		};

		// Toggle Pause/Pause buttons
		var handleOnPlay = function() {
			// check that controls exist
			if (videoControls) {
				// toggle icons
				d("videoPlayButton").style.display = "none";
				d("videoPauseButton").style.display = "inline-block";
			}
			// Fire event for play
			dispatchEvent("onPlay", {
				'threeSixtyViewerAPI': featureAPI
			});
		};

		// Toggle Pause/Pause buttons
		var handleOnPause = function() {
			// check that controls exist
			if (videoControls) {
				// toggle icons
				d("videoPlayButton").style.display = "inline-block";
				d("videoPauseButton").style.display = "none";
			}
			// Fire event for pause
			dispatchEvent("onPause", {
				'threeSixtyViewerAPI': featureAPI
			});
		};

		// Handle On ended event
		var handleOnEnded = function() {
			// If video is not on a loop, end autopan
			if (!videoLoop) {
				cancelPan();
			}
			// Fire event for ended
			dispatchEvent("onEnded", {
				'threeSixtyViewerAPI': featureAPI
			});
		};

		// Handle error event
		var handleOnError = function() {
			// Fire event for error
			dispatchEvent("onError", {
				'threeSixtyViewerAPI': featureAPI
			});
		};

		// Handle error event
		var handleOnTimeUpdate = function() {
			// Fire event for time update
			dispatchEvent("onTimeUpdate", {
				'threeSixtyViewerAPI': featureAPI
			});
		};

		// mute Video
		var muteVideo = function() {
			// mute video if video exists
			if (theVideo) {
				// mute
				theVideo.muted = true;
				// update boolean - track mute status
				videoMuted = true;
				// toggle icons
				d("videoMuteButton").style.display = "none";
				d("videoUnMuteButton").style.display = "inline-block";
			}
		};

		// unmute video
		var unMuteVideo = function() {
			// mute video if video exists
			if (theVideo) {
				// mute
				theVideo.muted = false;
				// update boolean - track mute status
				videoMuted = false;
				// toggle icons
				d("videoMuteButton").style.display = "inline-block";
				d("videoUnMuteButton").style.display = "none";
			}
		};

		// Add Controls to Viewer
		var buildControls = function() {
			// create new div
			var myContainerDiv = document.createElement("div");
			// Add id to div
			myContainerDiv.id = "VRControls";
			// Add to container
			container.appendChild(myContainerDiv);
			// Establish int to track positioning offset for fullscreen button
			var myDivOffset = 0;
			// Establish reference to new div element
			var myDiv;
			// Create new item
			myDiv = document.createElement("div");
			// add id
			myDiv.id = "fullscreenButton";
			// Add Class
			myDiv.className = "fullscreenButton";
			// add item to div
			myContainerDiv.appendChild(myDiv);
			// add reset button if not on Mobile
			if (!isMobile) {
				// Create new item
				myDiv = document.createElement("div");
				// add id
				myDiv.id = "resetButton";
				// Add Class
				myDiv.className = "resetButton";
				// add title to item
				myDiv.title = "reset position";
				// add item to div
				myContainerDiv.appendChild(myDiv);
			}
			else {
				// update offset so we position vr button correctly
				myDivOffset = 37;
			}
			// Check if iOS and hide fullscreen - fullscreen not available on iOS
			if (EB.API && (EB.API.os.ios || EB.API.os.ipad)) {
				// hide fullscreen button
				d("fullscreenButton").style.display = 'none';
				// increase offset to account for hidden fullscreen button
				myDivOffset += 37;
				// move VR button over
				setTimeout(function() {
					try {
						// get reference to vr button
						var vr = q(".a-enter-vr");
						// get right style of vr button
						var r = parseFloat(window.getComputedStyle(vr).getPropertyValue('right'));
						// update style of vr button
						vr.style.right = (r - myDivOffset) + "px";
					}
					catch (d) {}
				}, 750);
			}
			else {
				// move VR button over
				setTimeout(function() {
					try {
						// get reference to vr button
						var vr = q(".a-enter-vr");
						// get right style of vr button
						var r = parseFloat(window.getComputedStyle(vr).getPropertyValue('right'));
						// update style of vr button
						vr.style.right = (r - myDivOffset) + "px";
					}
					catch (d) {}
				}, 750);
			}

			// Add pan controls if requested
			if (enablePanControls && !isMobile) {
				// build pan controls
				myContainerDiv = document.createElement("div");
				// Add id to div
				myContainerDiv.id = "panControls";
				// Add to container
				container.appendChild(myContainerDiv);
				// build pan controls
				myDiv = document.createElement("div");
				// Add id to div
				myDiv.id = "panLeft";
				// add class Name
				myDiv.className = "panLeft";
				// Add to container
				myContainerDiv.appendChild(myDiv);
				// build pan controls
				myDiv = document.createElement("div");
				// Add id to div
				myDiv.id = "panRight";
				// add class Name
				myDiv.className = "panRight";
				// Add to container
				myContainerDiv.appendChild(myDiv);
				// build pan controls
				myDiv = document.createElement("div");
				// Add id to div
				myDiv.id = "panUp";
				// add class Name
				myDiv.className = "panUp";
				// Add to container
				myContainerDiv.appendChild(myDiv);
				// build pan controls
				myDiv = document.createElement("div");
				// Add id to div
				myDiv.id = "panDown";
				// add class Name
				myDiv.className = "panDown";
				// Add to container
				myContainerDiv.appendChild(myDiv);
			}
		};

		// Add listeners
		var addListeners = function() {
			// Add listener so we resize when we resize container
			window.addEventListener('resize', onResize);
			// Handle Fullscreen change event
			window.addEventListener('fullscreenchange', handleFSChange, false);
			window.addEventListener('mozfullscreenchange', handleFSChange, false);
			window.addEventListener('webkitfullscreenchange', handleFSChange, false);
			// Check if video option exists, create video tag
			if (videoSource !== '') {
				// Add listener for play and pause events
				theVideo.addEventListener("play", handleOnPlay, false);
				theVideo.addEventListener("pause", handleOnPause, false);
				theVideo.addEventListener("ended", handleOnEnded, false);
				theVideo.addEventListener("error", handleOnError, false);
				theVideo.addEventListener("timeupdate", handleOnTimeUpdate, false);
			}
		};

		// Add Scene listeners
		var addSceneListeners = function() {
			// Add listeners for reset button if adding pan
			if (!isMobile) {
				d('resetButton').addEventListener("click", resetCameraPosition, false);
				d('resetButton').addEventListener("mouseup", cancelPan, false);
			}
			// add listener for fullscreen button
			d('fullscreenButton').addEventListener("click", toggleFullscreen, false);
			d('fullscreenButton').addEventListener("mouseup", cancelPan, false);

			// Listen for click on scene
			aScene.addEventListener('mousedown', toggleDrag, false);
			aScene.addEventListener('mouseup', stopDrag, false);
			aScene.addEventListener('enter-vr', handleEnterVR, false);
			aScene.addEventListener('exit-vr', handleExitVR, false);

			// listen for device motion and hide panControls if on mobile
			if (typeof EBG.EventName !== "undefined" && isMobile) {
				//removeDeviceMotion = EB.addEventListener(EBG.EventName.DEVICE_MOTION, hidePanControls, false);
				EB.addEventListener(EBG.EventName.SCREEN_ORIENTATION, onScreenOrientationChange, false);
			}

			// Add listeners for controls if requested
			if (enablePanControls && !isMobile) {
				aScene.addEventListener("mouseup", cancelPan, false);
				d('panLeft').addEventListener("click", panViewLeft, false);
				d('panRight').addEventListener("click", panViewRight, false);
				d('panUp').addEventListener("click", panViewUp, false);
				d('panDown').addEventListener("click", panViewDown, false);

				d('panLeft').addEventListener("mousedown", function() {
					holdPan('l', 200);
				}, false);
				d('panRight').addEventListener("mousedown", function() {
					holdPan('r', 200);
				}, false);
				d('panUp').addEventListener("mousedown", function() {
					holdPan('u', 200);
				}, false);
				d('panDown').addEventListener("mousedown", function() {
					holdPan('d', 200);
				}, false);

				// Stop Panning if we mouse up on any of the buttons or the scene
				d('panLeft').addEventListener("mouseup", cancelPan, false);
				d('panRight').addEventListener("mouseup", cancelPan, false);
				d('panUp').addEventListener("mouseup", cancelPan, false);
				d('panDown').addEventListener("mouseup", cancelPan, false);
			}
			// check if key control was desired
			if (enableKeyControls) {
				// add listener to detect arrow keys
				window.addEventListener("keydown", detectKey, false);
			}
		};

		// handle key press detection
		var detectKey = function(e) {
			// Capture key event
			e = e || window.event;
			// Check if key is u/d/l/r and pan
			if (e.keyCode == '38') {
				panScene('u', true);
			}
			else if (e.keyCode == '40') {
				panScene('d', true);
			}
			else if (e.keyCode == '37') {
				panScene('l', true);
			}
			else if (e.keyCode == '39') {
				panScene('r', true);
			}
		};

		// handle drag event
		var toggleDrag = function() {
			// Cancel panning
			cancelPan();
			// Fire event for resize
			dispatchEvent("onDragStart", {
				'threeSixtyViewerAPI': featureAPI
			});
		};

		// Handle stop drag event
		var stopDrag = function() {
			// Establish reference to current scene
			var camera = q("#camera");
			// check that camera exists
			if (camera) {
				// Get scene coordinates
				var c = camera.getAttribute("rotation");
				// Update lat long for panning
				lon = Math.floor(c.y);
				// Check if lon is negative and make it a positive int within 360°
				if (lon < 0) {
					lon = Math.abs(lon + 360);
				}
				lat = Math.floor(c.x);
				// pan left one degree so we toggle orbit control
				panScene('l', false);
				// Fire event for resize
				dispatchEvent("onDragStop", {
					'threeSixtyViewerAPI': featureAPI
				});
			}
		};

		// Handle auto pan of scene
		var autoPan = function() {
			// Check if we should pan right
			if (enableAutoPan === "right") {
				holdPan('r');
			}
			// if not, pan left
			else {
				holdPan('l');
			}
		};

		// Handling panning scene when mouse is held down
		var holdPan = function(_dir, _duration) {
			// Establish duration of timeout
			var d = _duration || 200;
			// update boolean
			panning = true;
			// set timeout to fire pan scene ever 100ms
			holdPanTimeout = setTimeout(function() {
				// pan
				panScene(_dir, true);
				// fire again
				if (panning) {
					holdPan(_dir, d);
				}
			}, d);
		};

		// stop timeout
		var cancelPan = function() {
			// update boolean
			panning = false;
			// Clear timeout
			clearTimeout(holdPanTimeout);
		};

		// Pan the scene
		var panScene = function(_dir, _track) {
			// check how much the user has seen
			checkViewPercentage();
			// reset lon
			if (lon === 360) {
				lon = 0;
			}
			// Update appropriate coordinates for the correct direction
			if (_dir === 'l') {
				// increment longitude
				lon += 1;
				// Check if tracking
				if (_track) {
					// Fire event for pan event
					dispatchEvent("onPanLeft", {
						'threeSixtyViewerAPI': featureAPI
					});
				}
			}
			else if (_dir === 'r') {
				// increment longitude
				lon -= 1;
				// Check if tracking
				if (_track) {
					// Fire event for pan event
					dispatchEvent("onPanRight", {
						'threeSixtyViewerAPI': featureAPI
					});
				}
			}
			else if (_dir === 'u') {
				// increment latitude
				lat += 1;
				// Check if tracking
				if (_track) {
					// Fire event for pan event
					dispatchEvent("onPanUp", {
						'threeSixtyViewerAPI': featureAPI
					});
				}
			}
			else if (_dir === 'd') {
				// increment latitude
				lat -= 1;
				// Check if tracking
				if (_track) {
					// Fire event for pan event
					dispatchEvent("onPanDown", {
						'threeSixtyViewerAPI': featureAPI
					});
				}
			}
			// update camera rotation
			updateCameraRotation();
		};

		// reset Camera to initial position
		var resetCameraPosition = function() {
			EB.userActionCounter("click_resetCameraPosition");
			// Update lat/lon
			lon = 0;
			lat = 0;
			// Establish reference to current scene
			var camera = q("#camera");
			// Update rotate speed for a smooth reset
			camera.setAttribute('orbit-controls', 'rotateToSpeed', '.1');
			// update camera rotation
			updateCameraRotation();
			// Return rotate speed to normal
			setTimeout(function() {
				camera.setAttribute('orbit-controls', 'rotateToSpeed', '1');
			}, 1500);
			// pan left one degree so we toggle orbit control
			panScene('l', false);
			// Fire event for reset Event
			dispatchEvent("onReset", {
				'threeSixtyViewerAPI': featureAPI
			});
		};
		// rotate camera to initial position
		var updateCameraRotation = function() {
			// Establish reference to current scene
			var camera = q("#camera");
			// update lat
			lat = Math.max(-85, Math.min(85, lat));
			// Calculate Phi/Theta
			phi = THREE.Math.degToRad(-90 - lat);
			theta = THREE.Math.degToRad(-90 - lon);
			// update coordinates
			var x = Math.sin(phi) * Math.cos(theta);
			var y = Math.cos(phi);
			var z = Math.sin(phi) * Math.sin(theta);
			// Update camera using Three JS rotation
			camera.setAttribute('orbit-controls', 'rotateTo', x + " " + y + " " + z);
		};

		// reset viewing vars
		var resetViewPercent = function() {
			// Establish int to track viewed scene
			viewPercent = 0;
			// Establish Booleans to track how much has been viewed
			viewComplete = false;
			view25Percent = false;
			view50Percent = false;
			view75Percent = false;
		};

		// check how much the user has seen of the scene
		var checkViewPercentage = function() {
			// Check to see if we've hit the 0-90 quadrant
			if ((lon > 0 && lon < 89) && !view25Percent) {
				// update boolean so we only do this once
				view25Percent = true;
				// increment int
				viewPercent += 25;
				// fire event
				dispatchEvent("onView" + viewPercent + "Percent", {
					'threeSixtyViewerAPI': featureAPI
				});
			}
			// Check to see if we've hit the 90-180 quadrant
			else if ((lon > 90 && lon < 179) && !view50Percent) {
				// update boolean so we only do this once
				view50Percent = true;
				// increment int
				viewPercent += 25;
				// fire event
				dispatchEvent("onView" + viewPercent + "Percent", {
					'threeSixtyViewerAPI': featureAPI
				});
			}
			// Check to see if we've hit the 180-270 quadrant
			else if ((lon > 180 && lon < 269) && !view75Percent) {
				// update boolean so we only do this once
				view75Percent = true;
				// increment int
				viewPercent += 25;
				// fire event
				dispatchEvent("onView" + viewPercent + "Percent", {
					'threeSixtyViewerAPI': featureAPI
				});
			}
			// Check to see if we've hit the 270-360 quadrant
			else if ((lon > 270 && lon <= 360) && !viewComplete) {
				// update boolean so we only do this once
				viewComplete = true;
				// increment int
				viewPercent += 25;
				// fire event
				dispatchEvent("onView" + viewPercent + "Percent", {
					'threeSixtyViewerAPI': featureAPI
				});
			}
		};

		// Enter VR init
		var handleEnterVR = function() {
			EB.userActionCounter("click_EnterVR");
			// track VR Status
			inVR = true;
			// Check if we're already in FS
			// This will account for extra FS Change event
			if (isFS){
				// Toggle boolean
				isFS = !isFS;
			}
			// Hide controls
			if (videoControls) {
				d("videoControls").style.display = "none";
			}
			// Hide controls
			d("VRControls").style.display = "none";
			// Hide controls
			if (!isMobile) {
				d("panControls").style.display = "none";
			}
			// Fire event for entering VR
			dispatchEvent("onEnterVR", {
				'threeSixtyViewerAPI': featureAPI
			});
		};

		// Handle VR exit
		var handleExitVR = function() {
			EB.userActionCounter("click_ExitVR");
			// Check if VR - event fires twice
			if (inVR) {
				// update boolean
				inVR = false;
				// remove scene
				destroy();
				// add scene
				init();
				// Fire event for exiting VR
				dispatchEvent("onExitVR", {
					'threeSixtyViewerAPI': featureAPI
				});
			}
		};

		// handle canvas resize
		var onResize = function() {
			// Fire event for resize
			dispatchEvent("onResize", {
				'threeSixtyViewerAPI': featureAPI
			});
		};

		// request fullscreen
		var toggleFullscreen = function() {
			// check fullscreen status
			if (isFS) {
				// remove class for close fullscren
				d("fullscreenButton").classList.remove("closeFullscreenButton");
				// leave fullscreen
				killFullscreen();
			}
			else {
				// Add Class for close fullscreen
				d("fullscreenButton").classList.add("closeFullscreenButton");
				// enter fullscreen
				launchFullscreen();
			}
		};

		// Handle fullscreen change
		var handleFSChange = function() {
			// Toggle boolean
			isFS = !isFS;
			// Track fullscreen status
			if (!isFS) {
				// Return width and height to original size
				container.style = previousStyling;
			}
			// Resize Canvas
			onResize();
		};

		// request fullscreen
		var launchFullscreen = function() {
			EB.userActionCounter("click_EnterFullscreen");
			// Save original style object
			previousStyling = container.style;

			// request fullscreen
			if (container.requestFullscreen) {
				container.requestFullscreen();
				//theVideo.requestFullscreen();
			}
			else if (container.mozRequestFullScreen) {
				container.mozRequestFullScreen();
				//theVideo.mozRequestFullScreen();
			}
			else if (container.webkitRequestFullscreen) {
				container.webkitRequestFullscreen();
				//theVideo.webkitRequestFullscreen();
			}
			else if (container.msRequestFullscreen) {
				container.msRequestFullscreen();
				//theVideo.msRequestFullscreen();
			}
			// Update Container size
			expandContainer();
			// Fire event for onEnterFullscreen
			dispatchEvent("onEnterFullscreen", {
				'threeSixtyViewerAPI': featureAPI
			});
		};

		// Handle resizing container div
		var expandContainer = function() {
			// Get width and height of screen
			var w = screen.width;
			var h = screen.height;
			// Update container styles
			container.style.width = w + "px";
			container.style.height = h + "px";
			container.style.top = 0;
			container.style.left = 0;
			// resize scene
			aScene.resize();
		};

		// exit fullscreen
		var killFullscreen = function() {
			EB.userActionCounter("click_ExitFullscreen");
			if (document.exitFullscreen) {
				document.exitFullscreen();
			}
			else if (document.mozCancelFullScreen) {
				document.mozCancelFullScreen();
			}
			else if (document.webkitExitFullscreen) {
				document.webkitExitFullscreen();
			}
			else if (document.msExitFullscreen) {
				document.msExitFullscreen();
			}
			// Fire event for onEnterFullscreen
			dispatchEvent("onExitFullscreen", {
				'threeSixtyViewerAPI': featureAPI
			});
		};

		// Handle On Orientation Change
		var onScreenOrientationChange = function() {
			// Expand Screen if already FS
			if (isFS) {
				expandContainer();
			}
		};

		// completely remove scene
		var destroy = function() {
			// Check if panning and kill pan
			if (panning) {
				cancelPan();
			}

			for (var i = aScene.children.length - 1; i >= 0; i--) {
				var items = aScene.children[i];
				aScene.remove(items);
			}
			// Reset Vars
			theScene = '';
			theVideo = '';
			container.innerHTML = '';
			videoControls = false;
		};

		// public method for reset viewer to init position
		var resetViewer = function() {
			resetCameraPosition();
		};

		// public method for starting VR
		var initVR = function() {
			aScene.enterVR();
		};

		// public method for starting VR
		var exitVR = function() {
			aScene.exitVR();
		};

		// public method for Play Video
		var playVideo = function() {
			// Play video if video exists
			if (theVideo) {
				trackedVideo.playVideo();
			}
		};

		// public method for Play Video
		var pauseVideo = function() {
			// Play video if video exists
			if (theVideo) {
				// pause video
				theVideo.pause();
				// stop pan
				cancelPan();
			}
		};

		// public method for muting Video
		var mute = function() {
			muteVideo();
			return featureAPI;
		};

		// public method for muting Video
		var unMute = function() {
			unMuteVideo();
		};

		// public method for pan scene left
		var panViewLeft = function() {
			panScene('l', true);
		};

		// public method for pan scene right
		var panViewRight = function() {
			panScene('r', true);
		};

		// public method for pan scene up
		var panViewUp = function() {
			panScene('u', true);
		};

		// public method for pan scene down
		var panViewDown = function() {
			panScene('d', true);
		};

		// public method for launchFullscreen
		var enterFullscreen = function() {
			launchFullscreen();
		};

		// public method for killFullscreen
		var exitFullscreen = function() {
			killFullscreen();
		};

		// public method for adding 360 viewer
		var add360Viewer = function() {
			init();
		};

		// public method for removing 360 viewer
		var remove360Viewer = function() {
			destroy();
		};

		// DO NOT REMOVE THE FOLLOWING
		//=================================================
		// Utility Functions
		//-------------------------------------------------

		// Handle Dispatching events
		var dispatchEvent = function(_name, _data) {
			// Establish var for event
			var e = document.createEvent("Event");
			// Init Event
			e.initEvent(_name, true, false);
			// Loop through data
			for (var k in _data) {
				if (_data.hasOwnProperty(k)) {
					// Update event with data object
					e[k] = _data[k];
				}
			}
			// Dispatch Event
			window.dispatchEvent(e);
		};

		// Debugging
		var log = function() { // this is a closure-compiled version of the original code
			if (isDebug) {
				var b, a;
				b = Array.prototype.slice.call(arguments);
				a = new Date();
				a = scriptName + " (" + a.getFullYear() + "-" + a.getMonth() + "-" + a.getDate() + " " + (("0" + a.getHours()).slice(-2)) + ":" + (("0" + a.getMinutes()).slice(-2)) + ":" + (("0" + a.getSeconds()).slice(-2)) + "." + (("00" + a.getMilliseconds()).slice(-3)) + "): ";
				b.unshift(a);
				try {
					window.console && console.log && (console.log.apply ? console.log.apply(console, b) : console.log(b));
				}
				catch (d) {}
			}
		};

		// Track Ad Features - accepts interaction name and impression type
		var trackAdFeatures = function(_interaction, _noun) {
			// Establish var to track local mode
			var isServed;
			// Verify ad is not local
			try {
				isServed = EB._isServingMode();
			}
			catch (err) {}

			if (isServed && trackOnce) {
				try {
					// Update Boolean
					trackOnce = false;
					// Grab Version ID
					var adId = EB._adConfig.adId;
					// Establish reference for Version id
					var versionID;
					// Grab Version ID for mass versioning
					try {
						versionID = EB._adConfig.massVersioning.adVersions.split("_")[0];
					}
					catch (err) {
						versionID = "";
					}
					// Determine if tag is SAS
					var isSAS = (adId.toString().length >= 10);
					// Build Base Url for SAS and MDX
					var baseURL = (isSAS) ? "https://bs.serving-sys.com/Serving/adServer.bs?cn=display&c=19&pli=1073952795&adid=1073972634&ord=[timestamp]&rtu=-1&pcp=$$adID=" + adId + "|vID=" + versionID + "|interactionName=" : "https://bs.serving-sys.com/BurstingPipe/adServer.bs?cn=tf&c=19&mc=imp&pli=16478984&PluID=0&ord=%time%&rtu=-1&pcp=$$adID=" + adId + "|vId=" + versionID + "|interactionName=";
					// Build Tracking Pixel
					var trackingPixel = baseURL + _interaction + "|noun=" + _noun + "$$";
					// Fire Tracking Pixel by creating a new image.
					new Image().src = trackingPixel;

				}
				catch (err) {}
			}
		};

		// Set up Script
		var prepareFeature = function() {
			// Track Ad Feature
			trackAdFeatures("impression", "AF_" + scriptName);
			// init script
			init();
		};

		// set up script
		prepareFeature();

		// Return functions you want publically available
		featureAPI = {
			addEventListener: addEventListener,
			removeEventListener: removeEventListener,
			theVideo: theVideo,
			playVideo: playVideo,
			pauseVideo: pauseVideo,
			mute: mute,
			unMute: unMute,
			resetViewer: resetViewer,
			enterFullscreen: enterFullscreen,
			exitFullscreen: exitFullscreen,
			loadImage: loadImage,
			loadVideo: loadVideo,
			initVR: initVR,
			exitVR: exitVR,
			panViewLeft: panViewLeft,
			panViewRight: panViewRight,
			panViewDown: panViewDown,
			panViewUp: panViewUp,
			add360Viewer: add360Viewer,
			remove360Viewer: remove360Viewer,
			resetViewPercent: resetViewPercent,
			checkViewPercentage: checkViewPercentage,
		};

		return featureAPI;
	};
}());
